@@ -52,7 +52,7 @@ class TableauWorksheet:
         dataSegmentscp = copy.deepcopy(dataSegments)
         keys = list(dataSegmentscp.keys())
         for key in keys:
-            if key not in self._scraper.dataSegments:
+            if dataSegmentscp[key] is not None:
                 self._scraper.dataSegments[key] = dataSegmentscp[key]
 
     def getColumns(self) -> List[str]:
@@ -79,18 +79,23 @@ class TableauWorksheet:
             presModel = self._originalData["vqlCmdResponse"]["layoutStatus"]["applicationPresModel"]
             return [
                 t["fieldCaption"]
-                for t in tableauscraper.utils.getIndicesInfoVqlResponse(
-                    presModel, self.name, noSelectFilter=False
-                )
+                for t in tableauscraper.utils.getIndicesInfoVqlResponse(presModel, self.name, noSelectFilter=False)
             ]
         else:
             presModel = tableauscraper.utils.getPresModelVizData(
                 self._originalData)
+            if presModel is None:
+                presModel = tableauscraper.utils.getPresModelVizInfo(
+                    self._originalInfo)
+                indicesInfo = tableauscraper.utils.getIndicesInfoStoryPoint(
+                    presModel, self.name, noSelectFilter=False)
+            else:
+                indicesInfo = tableauscraper.utils.getIndicesInfo(
+                    presModel, self.name, noSelectFilter=False)
+
             return [
                 t["fieldCaption"]
-                for t in tableauscraper.utils.getIndicesInfo(
-                    presModel, self.name, noSelectFilter=False
-                )
+                for t in indicesInfo
             ]
 
     def getValues(self, column) -> List[str]:
@@ -116,11 +121,18 @@ class TableauWorksheet:
         else:
             presModel = tableauscraper.utils.getPresModelVizData(
                 self._originalData)
+            if presModel is None:
+                presModel = tableauscraper.utils.getPresModelVizInfo(
+                    self._originalInfo)
+                indicesInfo = tableauscraper.utils.getIndicesInfoStoryPoint(
+                    presModel, self.name, noSelectFilter=True)
+            else:
+                indicesInfo = tableauscraper.utils.getIndicesInfo(
+                    presModel, self.name, noSelectFilter=True)
+
             columnObj = [
                 t
-                for t in tableauscraper.utils.getIndicesInfo(
-                    presModel, self.name, noSelectFilter=True
-                )
+                for t in indicesInfo
                 if t["fieldCaption"] == column
             ]
             if len(columnObj) == 0:
@@ -134,12 +146,44 @@ class TableauWorksheet:
                 return []
             return frameData[frameDataKeys[0]]
 
+    def getTupleIds(self, column) -> List[int]:
+        if self.cmdResponse:
+            presModel = self._originalData["vqlCmdResponse"]["layoutStatus"]["applicationPresModel"]
+            columnObj = [
+                t
+                for t in tableauscraper.utils.getIndicesInfoVqlResponse(
+                    presModel, self.name, noSelectFilter=True, noFieldCaption=True
+                )
+                if t["fn"] == "[system:visual].[tuple_id]"
+            ]
+            if len(columnObj) == 0:
+                return []
+
+            return columnObj[0]["tupleIds"]
+        else:
+            presModel = tableauscraper.utils.getPresModelVizData(
+                self._originalData)
+            columnObj = [
+                t
+                for t in tableauscraper.utils.getIndicesInfo(
+                    presModel, self.name, noSelectFilter=True, noFieldCaption=True
+                )
+                if t["fn"] == "[system:visual].[tuple_id]"
+            ]
+            if len(columnObj) == 0:
+                return []
+            return columnObj[0]["tupleIds"]
+
     def select(self, column, value):
         values = self.getValues(column)
+        tupleIds = self.getTupleIds(column)
         try:
             index = values.index(value)
-            r = tableauscraper.api.select(
-                self._scraper, self.name, [index + 1])
+            if len(tupleIds) > index:
+                index = tupleIds[index]
+            else:
+                index = index + 1
+            r = tableauscraper.api.select(self._scraper, self.name, [index])
             self.updateFullData(r)
             return tableauscraper.dashboard.getWorksheetsCmdResponse(self._scraper, r)
         except ValueError:
