@@ -2,29 +2,43 @@ import json
 import time
 import requests
 
+
 def setSession(scraper):
     scraper.session = requests.Session()
 
-def getTableauVizForSession(session, url):
-    r = session.get(url, params={":embed": "y", ":showVizHome": "no"})
+
+def getTableauVizForSession(scraper, session, url):
+    r = session.get(url, params={
+        ":embed": "y",
+        ":showVizHome": "no"},
+        verify=scraper.verify
+    )
     return r.text
 
-def getTableauViz(session, url):
-    r = session.get(url, params={":embed": "y", ":showVizHome": "no"})
+
+def getTableauViz(scraper, session, url):
+    r = session.get(url, params={
+        ":embed": "y",
+        ":showVizHome": "no"
+    },
+        verify=scraper.verify
+    )
     return r.text
 
-def getSessionUrl(session, url):
-    r = session.get(url)
+
+def getSessionUrl(scraper, session, url):
+    r = session.get(url, verify=scraper.verify)
     return r.text
 
+
 def getTableauData(scraper):
     dataUrl = f'{scraper.host}{scraper.tableauData["vizql_root"]}/bootstrapSession/sessions/{scraper.tableauData["sessionid"]}'
-
     r = scraper.session.post(
         dataUrl,
         data={
             "sheet_id": scraper.tableauData["sheetId"],
         },
+        verify=scraper.verify
     )
     scraper.lastActionTime = time.time()
     return r.text
@@ -41,6 +55,7 @@ def select(scraper, worksheetName, selection):
     r = scraper.session.post(
         f'{scraper.host}{scraper.tableauData["vizql_root"]}/sessions/{scraper.tableauData["sessionid"]}/commands/tabdoc/select',
         data=payload,
+        verify=scraper.verify
     )
     return r.json()
 
@@ -55,6 +70,7 @@ def setParameterValue(scraper, parameterName, value):
     r = scraper.session.post(
         f'{scraper.host}{scraper.tableauData["vizql_root"]}/sessions/{scraper.tableauData["sessionid"]}/commands/tabdoc/set-parameter-value',
         files=payload,
+        verify=scraper.verify
     )
     return r.json()
 
